/*
How do I customize the MFC idle time processing?
In my opinion, MFC's idle processing happens too often. Unfortunately, we can't change it 
because it might break apps when not necessary. But. . . we have built into the code a way 
for applications to customize the times in which they "enter idle". The secret? CWinThread::IsIdleMessage!
Here's an example that disables idle entry for mouse and timer messages:

BOOL CMyApp::IsIdleMessage(MSG* pMsg)
{
    if (!CWinApp::IsIdleMessage(pMsg))
    return FALSE;
    if ((pMsg->message >= WM_MOUSEFIRST && 
        pMsg->message <= WM_MOUSELAST) ||
        (pMsg->message >= WM_NCMOUSEMOVE && 
        pMsg->message <= WM_NCMBUTTONDBLCLK))
            return FALSE;
    if (pMsg->message == WM_TIMER)
        return FALSE;
    // Otherwise msg must kick in idle processing...
    return TRUE;
}
Now, what would happen if you had a clock on your status bar, and were relying on a 
WM_TIMER to kick in a call to your update handler for that pane? Well...as you might 
have guessed, the above code would break you. There is an easy way to fix that, as 
there is a backdoor method of forcing an idle to kick in: WM_KICKIDLE, defined in 
afxpriv.h, can be used to make this happen.
So, in your timer handler (most likely a handler in your derived CStatusBar class), you 
would add some code that does:
if (TimeIsDifferentEnough())
    PostMessage(WM_KICKIDLE);
If you take a look at CWinThread::IsIdleMessage, you'll see that we go through a fair 
amount of pain to avoid unnecessary OnIdle calls, like checking for mouse moves to
 the same location as last time. (I don't know why Windows NT does that, but it 
doesevery time the caret flashes.) We are doing just about everything we can without 
breaking backward compatibility.
*/