
NTSTATUS UsbBulkDevice::Read(KIrp I) 
{
    NTSTATUS status = STATUS_INSUFFICIENT_RESOURCES;

    Tracer << "UsbBulkDevice::Read() Entry\n";

	// Declare a memory object
	KMemory Mem(I.Mdl());

    ULONG dwTotalSize = I.ReadSize(CURRENT);
	ULONG dwMaxSize = m_Pipe0.MaximumTransferSize();

	// If the total requested read size is greater than the Maximum Transfer
	// Size for the Pipe, request to read only the Maximum Transfer Size since
	// the bus driver will fail an URB with a TransferBufferLength of greater
	// than the Maximum Transfer Size. 
	if ( dwTotalSize > dwMaxSize )
	{
		ASSERT(dwMaxSize);
		dwTotalSize = dwMaxSize;
	}

    ULONG dwBytesRead = 0;

    // Create an URB to do actual Bulk read from Pipe0
    PURB pUrb = m_Pipe0.BuildBulkTransfer(
							Mem,		  // Where is data coming from?
							dwTotalSize,  // How much data to read?
							TRUE,         // direction (TRUE = IN)
							NULL	      // Link to next URB
							);

	if ( pUrb != NULL)
	{
	    // Submit the URB to our USB device, synchronously - say less is OK
		pUrb->UrbBulkOrInterruptTransfer.TransferFlags =
				(USBD_TRANSFER_DIRECTION_IN | USBD_SHORT_TRANSFER_OK);

        status = m_Pipe0.SubmitUrb(pUrb, NULL, NULL);

        if ( NT_SUCCESS(status) ) 
        {
	            dwBytesRead = pUrb->UrbBulkOrInterruptTransfer.TransferBufferLength;

				if (dwBytesRead > 0) 
					Tracer << "Read() got " << dwTotalSize	<< " bytes from USB\n";
	    }

		delete pUrb;
	}

	
    I.Information() = dwBytesRead;
    return I.PnpComplete(this, status, IO_NO_INCREMENT);
}




===================================================================================
////////////////////////////////////////////////////////////////////////
// UsbThermometer::ReadRamSynch
//
// Perform a synchronous read of the thermometer's RAM
// 
// Input
//		offset		RAM address to read
//
// Output
//		*output		data read from offset
//
NTSTATUS UsbThermometer::ReadRamSynch(UCHAR offset, UCHAR* output)
{
	PURB pUrb;
	NTSTATUS status;
	UCHAR buffer[8];

	pUrb = m_Usb.BuildVendorRequest(
				buffer,							// transfer buffer
				8,								// transfer buffer size
				0,								// request reserved bits
				2,								// request
				offset,							// Value
				TRUE,							// In
				FALSE,							// Short Ok
				NULL,							// link urb
				0,								// index
				URB_FUNCTION_VENDOR_ENDPOINT	// function
				);

	if ( pUrb != NULL )
	{
		status = m_Usb.SubmitUrb(pUrb);
		
		if ( !NT_SUCCESS(status) )
		{
			t << "Synch read error: status=" << ULONG(status) 
				<< " Urb status=" << ULONG(pUrb->UrbHeader.Status) << "\n";
		}
		else
			*output = buffer[1];

		delete pUrb;
	}
	else
	{
		status = STATUS_INSUFFICIENT_RESOURCES;
	}

	return status;
}


////////////////////////////////////////////////////////////////////////
// UsbThermometer::WriteRamSynch
//
// Peform a synchronous write of the thermometer's RAM
//
// Input
//		offset		RAM address to write
//		value		value to write to RAM
//
NTSTATUS UsbThermometer::WriteRamSynch(UCHAR offset, UCHAR value)
{
	PURB pUrb;
	NTSTATUS status;

	UCHAR buffer[8];
	memset(buffer, 0, 8);

	pUrb = m_Usb.BuildVendorRequest(
				buffer,							// transfer buffer
				8,								// transfer buffer size
				0,								// request reserved bits
				3,								// request
				offset,							// Value
				TRUE,							// In (??)
				TRUE,							// Short Ok
				NULL,							// link urb
				value,							// index
				URB_FUNCTION_VENDOR_ENDPOINT	// function
				);

	status = m_Usb.SubmitUrb(pUrb);
	delete pUrb;

	return status;
}